@@ -69,6 +69,14 @@
 
 #include <stdio.h>
 #include <string.h>
+#ifndef VMS
+#include <sys/time.h>
+#else
+#include <time.h>
+#endif
+#if defined(AIXV3) || defined(_SYSTYPE_SVR4) || defined(ibm) || defined __QNX__
+#include <sys/select.h>
+#endif
 #include "twm.h"
 #ifdef VMS
 #include <decw$include/Xatom.h>
@@ -84,6 +92,7 @@
 #include "events.h"
 #include "menus.h"
 #include "screen.h"
+#include "icons.h"
 #include "iconmgr.h"
 
 #define gray_width 2
@@ -101,14 +110,23 @@ int AddingH;
 static int PlaceX = 50;
 static int PlaceY = 50;
 static void CreateWindowTitlebarButtons();
-void ClickToFocusGrab   ();
-void ClickToFocusUngrab ();
+
+static void		splitWindowRegionEntry ();
+static WindowEntry	*findWindowEntry ();
+static WindowEntry	*prevWindowEntry ();
+static void		mergeWindowEntries ();
 
 char NoName[] = "Untitled"; /* name if no name is specified */
 int  resizeWhenAdd;
 
 extern Atom _OL_WIN_ATTR;
+extern int captive;
 
+#if defined(__hpux) && !defined(_XPG4_EXTENDED)
+#   define FDSET int*
+#else
+#   define FDSET fd_set*
+#endif
 /************************************************************************
  *
  *  Procedure:
@@ -179,17 +197,42 @@ IconMgr *iconp;
     unsigned long valuemask;		/* mask for create windows */
     XSetWindowAttributes attributes;	/* attributes for create windows */
     int width, height;			/* tmp variable */
+#ifdef NO_LOCALE
     Atom actual_type;
     int actual_format;
     unsigned long nitems, bytesafter;
+#endif /* NO_LOCALE */
     int ask_user;		/* don't know where to put the window */
     int gravx, gravy;			/* gravity signs for positioning */
     int namelen;
     int bw2;
+#ifdef X11R6
+    short saved_x, saved_y, restore_icon_x, restore_icon_y;
+    unsigned short saved_width, saved_height;
+    Bool restore_iconified = 0;
+    Bool restore_icon_info_present = 0;
+    int restoredFromPrevSession;
+    Bool width_ever_changed_by_user;
+    Bool height_ever_changed_by_user;
+    int saved_occupation; /* <== [ Matthew McNeill Feb 1997 ] == */
+#endif
+    int		found;
+#ifndef VMS
+    fd_set	mask;
+    int		fd;
+    struct timeval timeout;
+#endif
+#ifdef I18N
+    XRectangle ink_rect;
+    XRectangle logical_rect;
+#endif
 
 #ifdef DEBUG
     fprintf(stderr, "AddWindow: w = 0x%x\n", w);
 #endif
+
+    if (!captive && RedirectToCaptive (w)) return (NULL);
+
     /* allocate space for the twm window */
     tmp_win = (TwmWindow *)calloc(1, sizeof(TwmWindow));
     if (tmp_win == 0)
@@ -203,14 +246,51 @@ IconMgr *iconp;
     tmp_win->iconmgr = iconm;
     tmp_win->iconmgrp = iconp;
     tmp_win->cmaps.number_cwins = 0;
+    tmp_win->savegeometry.width = -1;
 
     XSelectInput(dpy, tmp_win->w, PropertyChangeMask);
     XGetWindowAttributes(dpy, tmp_win->w, &tmp_win->attr);
+#ifndef NO_LOCALE
+    tmp_win->name = GetWMPropertyString(tmp_win->w, XA_WM_NAME);
+#else /* NO_LOCALE */
     XFetchName(dpy, tmp_win->w, &tmp_win->name);
+#endif /* NO_LOCALE */
     tmp_win->class = NoClass;
     XGetClassHint(dpy, tmp_win->w, &tmp_win->class);
     FetchWmProtocols (tmp_win);
     FetchWmColormapWindows (tmp_win);
+
+#ifdef X11R6
+    if (GetWindowConfig (tmp_win,
+	&saved_x, &saved_y, &saved_width, &saved_height,
+	&restore_iconified, &restore_icon_info_present,
+	&restore_icon_x, &restore_icon_y,
+	&width_ever_changed_by_user, &height_ever_changed_by_user,
+	&saved_occupation)) /* <== [ Matthew McNeill Feb 1997 ] == */
+    {
+	tmp_win->attr.x = saved_x;
+	tmp_win->attr.y = saved_y;
+
+	tmp_win->widthEverChangedByUser = width_ever_changed_by_user;
+	tmp_win->heightEverChangedByUser = height_ever_changed_by_user;
+	
+	if (width_ever_changed_by_user)
+	    tmp_win->attr.width = saved_width;
+
+	if (height_ever_changed_by_user)
+	    tmp_win->attr.height = saved_height;
+
+	restoredFromPrevSession = 1;
+    }
+    else
+    {
+	tmp_win->widthEverChangedByUser = False;
+	tmp_win->heightEverChangedByUser = False;
+
+	restoredFromPrevSession = 0;
+    }
+#endif
+
     /*
      * do initial clip; should look at window gravity
      */
@@ -220,6 +300,30 @@ IconMgr *iconp;
       tmp_win->attr.height = Scr->MaxWindowHeight;
 
     tmp_win->wmhints = XGetWMHints(dpy, tmp_win->w);
+
+#ifdef X11R6
+    if (tmp_win->wmhints)
+    {
+	if (restore_iconified)
+	{
+	    tmp_win->wmhints->initial_state = IconicState;
+	    tmp_win->wmhints->flags |= StateHint;
+	}
+
+	if (restore_icon_info_present)
+	{
+	    tmp_win->wmhints->icon_x = restore_icon_x;
+	    tmp_win->wmhints->icon_y = restore_icon_y;
+	    tmp_win->wmhints->flags |= IconPositionHint;
+	}
+    }
+#endif
+
+    if (tmp_win->wmhints) tmp_win->wmhints->input = True;
+				/* CL: Having with not willing focus
+				cause problems with AutoSqueeze and a few others
+				things. So I suppress it. And the whole focus thing
+				is buggy anyway */
     if (tmp_win->wmhints && !(tmp_win->wmhints->flags & InputHint))
 	tmp_win->wmhints->input = True;
     if (tmp_win->wmhints && (tmp_win->wmhints->flags & WindowGroupHint)) 
@@ -234,6 +338,9 @@ IconMgr *iconp;
 
     tmp_win->transient = Transient(tmp_win->w, &tmp_win->transientfor);
 
+#ifdef X11R6
+    tmp_win->nameChanged = 0;
+#endif
     if (tmp_win->name == NULL)
 	tmp_win->name = NoName;
     if (tmp_win->class.res_name == NULL)
@@ -274,6 +381,34 @@ IconMgr *iconp;
 	(short)(int) LookInList(Scr->IconifyByUn, tmp_win->full_name,
 	    &tmp_win->class);
 
+    if (LookInList (Scr->UnmapByMovingFarAway, tmp_win->full_name, &tmp_win->class))
+	tmp_win->UnmapByMovingFarAway = True;
+    else
+	tmp_win->UnmapByMovingFarAway = False;
+
+    if (LookInList (Scr->DontSetInactive, tmp_win->full_name, &tmp_win->class))
+	tmp_win->DontSetInactive = True;
+    else
+	tmp_win->DontSetInactive = False;
+
+    if (LookInList (Scr->AutoSqueeze, tmp_win->full_name, &tmp_win->class))
+	tmp_win->AutoSqueeze = True;
+    else
+	tmp_win->AutoSqueeze = False;
+
+    if (LookInList (Scr->StartSqueezed, tmp_win->full_name, &tmp_win->class))
+	tmp_win->StartSqueezed = True;
+    else
+	tmp_win->StartSqueezed = False;
+
+    if (tmp_win->transient || tmp_win->group) {
+	TwmWindow *t;
+	for (t = Scr->TwmRoot.next; t != NULL; t = t->next) {
+	    if (tmp_win->transient && (tmp_win->transientfor == t->w)) break;
+	    if (tmp_win->group     && (tmp_win->group        == t->w)) break;
+	}
+	if (t) tmp_win->UnmapByMovingFarAway = t->UnmapByMovingFarAway;
+    }
     if (Scr->WindowRingAll ||
 	LookInList(Scr->WindowRingL, tmp_win->full_name, &tmp_win->class)) {
 	if (Scr->Ring) {
@@ -366,7 +501,22 @@ IconMgr *iconp;
     }
 
     GetWindowSizeHints (tmp_win);
-    GetGravityOffsets (tmp_win, &gravx, &gravy);
+
+#ifdef X11R6
+    if (restoredFromPrevSession)
+    {
+	/*
+	 * When restoring window positions from the previous session,
+	 * we always use NorthWest gravity.
+	 */
+
+	gravx = gravy = -1;
+    }
+    else
+#endif
+    {
+	GetGravityOffsets (tmp_win, &gravx, &gravy);
+    }
 
     /*
      * Don't bother user if:
@@ -386,12 +536,39 @@ IconMgr *iconp;
 	  tmp_win->attr.x != 0 || tmp_win->attr.y != 0)))
       ask_user = FALSE;
 
+    /*===============[ Matthew McNeill 1997 ]==========================*
+     * added the occupation parameter to this function call so that the 
+     * occupation can be set up in a specific state if desired 
+     * (restore session for example)
+     */
+
+#ifdef X11R6
+    if (restoredFromPrevSession) {
+      SetupOccupation (tmp_win, saved_occupation);
+    } else {
+      SetupOccupation (tmp_win, 0);
+    }
+#else
+      SetupOccupation (tmp_win, 0);
+#endif      
+    /*=================================================================*/
+
+    tmp_win->frame_width  = tmp_win->attr.width  + 2 * tmp_win->frame_bw3D;
+    tmp_win->frame_height = tmp_win->attr.height + 2 * tmp_win->frame_bw3D +
+			    tmp_win->title_height;
+    ConstrainSize (tmp_win, &tmp_win->frame_width, &tmp_win->frame_height);
+    if (PlaceWindowInRegion (tmp_win, &(tmp_win->attr.x), &(tmp_win->attr.y))) {
+	ask_user = False;
+    }
     /*
      * do any prompting for position
      */
-    SetupOccupation (tmp_win);
 
+#ifdef X11R6
+    if (HandlingEvents && ask_user && !restoredFromPrevSession) {
+#else
     if (HandlingEvents && ask_user) {
+#endif
       if ((Scr->RandomPlacement == RP_ALL) ||
           ((Scr->RandomPlacement == RP_UNMAPPED) &&
 	   ((tmp_win->wmhints && (tmp_win->wmhints->initial_state == IconicState)) ||
@@ -463,22 +640,41 @@ IconMgr *iconp;
 		    break;
 	    }
 
+#ifdef I18N
+	    XmbTextExtents(Scr->SizeFont.font_set,
+			   tmp_win->name, namelen,
+			   &ink_rect, &logical_rect);
+	    width = SIZE_HINDENT + ink_rect.width;
+	    height = logical_rect.height + SIZE_VINDENT * 2;
+	    XmbTextExtents(Scr->SizeFont.font_set,
+			   ": ", 2,  &logical_rect, &logical_rect);
+	    Scr->SizeStringOffset = width + logical_rect.width;
+#else	    
 	    width = (SIZE_HINDENT + XTextWidth (Scr->SizeFont.font,
 						tmp_win->name, namelen));
 	    height = Scr->SizeFont.height + SIZE_VINDENT * 2;
 	    Scr->SizeStringOffset = width + XTextWidth (Scr->SizeFont.font, ": ", 2);
+#endif	    
 	    
 	    XResizeWindow (dpy, Scr->SizeWindow, Scr->SizeStringOffset +
-				Scr->SizeStringWidth, height);
+				Scr->SizeStringWidth + SIZE_HINDENT, height);
 	    XMapRaised(dpy, Scr->SizeWindow);
 	    InstallRootColormap();
-
+#ifdef I18N
+	    FB(Scr->DefaultC.fore, Scr->DefaultC.back);
+	    XmbDrawImageString (dpy, Scr->SizeWindow, Scr->SizeFont.font_set,
+				Scr->NormalGC, SIZE_HINDENT,
+				SIZE_VINDENT + Scr->SizeFont.ascent,
+				tmp_win->name, namelen);
+#else
 	    FBF(Scr->DefaultC.fore, Scr->DefaultC.back,
 		Scr->SizeFont.font->fid);
+
 	    XDrawImageString (dpy, Scr->SizeWindow, Scr->NormalGC,
 			      SIZE_HINDENT,
 			      SIZE_VINDENT + Scr->SizeFont.font->ascent,
 			      tmp_win->name, namelen);
+#endif	    
 
 	    AddingW = tmp_win->attr.width + bw2 + 2 * tmp_win->frame_bw3D;
 	    AddingH = tmp_win->attr.height + tmp_win->title_height +
@@ -487,14 +683,40 @@ IconMgr *iconp;
 		MoveOutline(Scr->Root, AddingX, AddingY, AddingW, AddingH,
 			    tmp_win->frame_bw, tmp_win->title_height + tmp_win->frame_bw3D);
 
+#ifdef I18N
+	    XmbDrawImageString (dpy, Scr->SizeWindow, Scr->SizeFont.font_set,
+				Scr->NormalGC, width,
+				SIZE_VINDENT + Scr->SizeFont.ascent, ": ", 2);
+#else	    
 	    XDrawImageString (dpy, Scr->SizeWindow, Scr->NormalGC, width,
 				SIZE_VINDENT + Scr->SizeFont.font->ascent, ": ", 2);
+#endif	  
 	    DisplayPosition (tmp_win, AddingX, AddingY);
+
+	    tmp_win->frame_width  = AddingW;
+	    tmp_win->frame_height = AddingH;
 	    /*SetFocus ((TwmWindow *) NULL, CurrentTime);*/
 	    while (TRUE)
 		{
-		XMaskEvent(dpy, ButtonPressMask | PointerMotionMask, &event);
-
+#ifndef VMS			/* I'll try to implement this later.  RL */
+		if (Scr->OpenWindowTimeout) {
+		    fd = ConnectionNumber (dpy);
+		    while (!XCheckMaskEvent (dpy, ButtonMotionMask | ButtonPressMask, &event)) {
+			FD_ZERO (&mask);
+			FD_SET  (fd, &mask);
+			timeout.tv_sec  = Scr->OpenWindowTimeout;
+			timeout.tv_usec = 0;
+			found = select (fd + 1, (FDSET)&mask, (FDSET)0, (FDSET)0, &timeout);
+			if (found == 0) break;
+		    }
+		    if (found == 0) break;
+		} else {
+#else
+		{
+#endif
+		    found = 1;
+		    XMaskEvent(dpy, ButtonPressMask | PointerMotionMask, &event);
+		}
 		if (event.type == MotionNotify) {
 		    /* discard any extra motion events before a release */
 		    while(XCheckMaskEvent(dpy,
@@ -508,7 +730,10 @@ IconMgr *iconp;
 		if (event.type == ButtonPress) {
 		  AddingX = event.xbutton.x_root;
 		  AddingY = event.xbutton.y_root;
-		  
+
+		  TryToGrid (tmp_win, &AddingX, &AddingY);
+		  if (Scr->PackNewWindows) TryToPack (tmp_win, &AddingX, &AddingY);
+
 		  /* DontMoveOff prohibits user form off-screen placement */
 		  if (Scr->DontMoveOff)	
   		    {
@@ -544,6 +769,8 @@ IconMgr *iconp;
 		XQueryPointer(dpy, Scr->Root, &JunkRoot, &JunkChild,
 		    &JunkX, &JunkY, &AddingX, &AddingY, &JunkMask);
 
+		TryToGrid (tmp_win, &AddingX, &AddingY);
+		if (Scr->PackNewWindows) TryToPack (tmp_win, &AddingX, &AddingY);
 		if (Scr->DontMoveOff)
 		{
 		    int AddingR, AddingB;
@@ -573,16 +800,32 @@ IconMgr *iconp;
 		DisplayPosition (tmp_win, AddingX, AddingY);
 	    }
 
+	  if (found) {
 	    if (event.xbutton.button == Button2) {
 		int lastx, lasty;
 
+#ifdef I18N
+		XmbTextExtents(Scr->SizeFont.font_set,
+			       ": ", 2,  &logical_rect, &logical_rect);
+		Scr->SizeStringOffset = width + logical_rect.width;
+
+		XResizeWindow (dpy, Scr->SizeWindow, Scr->SizeStringOffset +
+			       Scr->SizeStringWidth + SIZE_HINDENT, height);
+
+		XmbDrawImageString(dpy, Scr->SizeWindow, Scr->SizeFont.font_set,
+				   Scr->NormalGC, width,
+				   SIZE_VINDENT + Scr->SizeFont.ascent, ": ", 2);
+#else		
 		Scr->SizeStringOffset = width +
 		  XTextWidth(Scr->SizeFont.font, ": ", 2);
 		XResizeWindow (dpy, Scr->SizeWindow, Scr->SizeStringOffset +
-			       Scr->SizeStringWidth, height);
+			       Scr->SizeStringWidth + SIZE_HINDENT, height);
+
 		XDrawImageString (dpy, Scr->SizeWindow, Scr->NormalGC, width,
 				  SIZE_VINDENT + Scr->SizeFont.font->ascent,
 				  ": ", 2);
+#endif
+		
 		if (0/*Scr->AutoRelativeResize*/) {
 		    int dx = (tmp_win->attr.width / 4);
 		    int dy = (tmp_win->attr.height / 4);
@@ -674,7 +917,7 @@ IconMgr *iconp;
 	    {
 		XMaskEvent(dpy, ButtonReleaseMask, &event);
 	    }
-
+	  }
 	    MoveOutline(Scr->Root, 0, 0, 0, 0, 0, 0);
 	    XUnmapWindow(dpy, Scr->SizeWindow);
 	    UninstallRootColormap();
@@ -710,19 +953,41 @@ IconMgr *iconp;
 
     tmp_win->title_width = tmp_win->attr.width;
 
+#ifdef I18N
+    XmbTextExtents(Scr->TitleBarFont.font_set, tmp_win->name, namelen,
+		   &ink_rect, &logical_rect);
+    
+    tmp_win->name_width = logical_rect.width;
+#else    
     tmp_win->name_width = XTextWidth(Scr->TitleBarFont.font, tmp_win->name,
 				     namelen);
+#endif    
 
+#ifndef NO_LOCALE
+    tmp_win->icon_name = GetWMPropertyString(tmp_win->w, XA_WM_ICON_NAME);
+#else /* NO_LOCALE */
     if (XGetWindowProperty (dpy, tmp_win->w, XA_WM_ICON_NAME, 0L, 200L, False,
 			    XA_STRING, &actual_type, &actual_format, &nitems,
 			    &bytesafter,(unsigned char **)&tmp_win->icon_name))
 	tmp_win->icon_name = tmp_win->name;
+#endif /* NO_LOCALE */
 
     if (tmp_win->icon_name == NULL)
 	tmp_win->icon_name = tmp_win->name;
+#ifdef CLAUDE
+    else if ((strlen (tmp_win->icon_name) > 11) &&
+	    (strncmp (tmp_win->icon_name, "Netscape: ", 10) == 0)) {
+	char *tmp;
+
+	tmp = strdup (tmp_win->icon_name + 10);
+	XFree ((char*) tmp_win->icon_name);
+	tmp_win->icon_name = tmp;
+    }
+#endif
 
     if (tmp_win->old_bw) XSetWindowBorderWidth (dpy, tmp_win->w, 0);
 
+    tmp_win->squeezed = FALSE;
     tmp_win->iconified = FALSE;
     tmp_win->isicon = FALSE;
     tmp_win->icon_on = FALSE;
@@ -739,6 +1004,13 @@ IconMgr *iconp;
     if (XGetGeometry(dpy, tmp_win->w, &JunkRoot, &JunkX, &JunkY,
 		     &JunkWidth, &JunkHeight, &JunkBW, &JunkDepth) == 0)
     {
+	TwmWindow *prev = tmp_win->ring.prev, *next = tmp_win->ring.next;
+
+	if (prev) prev->ring.next = next;
+	if (next) next->ring.prev = prev;
+	if (Scr->Ring == tmp_win) Scr->Ring = (next != tmp_win ? next : (TwmWindow *) NULL);
+	if (!Scr->Ring || Scr->RingLeader == tmp_win) Scr->RingLeader = Scr->Ring;
+
 	free((char *)tmp_win);
 	XUngrabServer(dpy);
 	return(NULL);
@@ -790,9 +1062,10 @@ IconMgr *iconp;
 
     ConstrainSize (tmp_win, &tmp_win->frame_width, &tmp_win->frame_height);
 
-    valuemask = CWBackPixmap | CWBorderPixel | CWCursor | CWEventMask;
+    valuemask = CWBackPixmap | CWBorderPixel | CWCursor | CWEventMask | CWBackPixel;
     attributes.background_pixmap = None;
-    attributes.border_pixel = tmp_win->borderC.back;
+    attributes.border_pixel = tmp_win->border_tile.back;
+    attributes.background_pixel = tmp_win->border_tile.back;
     attributes.cursor = Scr->FrameCursor;
     attributes.event_mask = (SubstructureRedirectMask | 
 			     ButtonPressMask | ButtonReleaseMask |
@@ -900,7 +1173,9 @@ IconMgr *iconp;
 	tmp_win->wShaped = boundingShaped;
     }
 
-    if (!tmp_win->iconmgr)
+    if (!tmp_win->iconmgr &&
+	(tmp_win->w != Scr->workSpaceMgr.workspaceWindow.w) &&
+	(tmp_win->w != Scr->workSpaceMgr.occupyWindow.w))
 	XAddToSaveSet(dpy, tmp_win->w);
 	
     XReparentWindow(dpy, tmp_win->w, tmp_win->frame, tmp_win->frame_bw3D,
@@ -926,41 +1201,47 @@ IconMgr *iconp;
     {
 	GrabButtons(tmp_win);
 	GrabKeys(tmp_win);
-	if (Scr->ClickToFocus &&
-	    tmp_win->wmhints && tmp_win->wmhints->input &&
-	    (w != Scr->workSpaceMgr.workspaceWindow.w) &&
-	    (w != Scr->workSpaceMgr.occupyWindow.w)) ClickToFocusGrab (tmp_win);
     }
 
     (void) AddIconManager(tmp_win);
 
-    XSaveContext(dpy, tmp_win->w, TwmContext, (caddr_t) tmp_win);
-    XSaveContext(dpy, tmp_win->w, ScreenContext, (caddr_t) Scr);
-    XSaveContext(dpy, tmp_win->frame, TwmContext, (caddr_t) tmp_win);
-    XSaveContext(dpy, tmp_win->frame, ScreenContext, (caddr_t) Scr);
+    XSaveContext(dpy, tmp_win->w, TwmContext, (XPointer) tmp_win);
+    XSaveContext(dpy, tmp_win->w, ScreenContext, (XPointer) Scr);
+    XSaveContext(dpy, tmp_win->frame, TwmContext, (XPointer) tmp_win);
+    XSaveContext(dpy, tmp_win->frame, ScreenContext, (XPointer) Scr);
 
     if (tmp_win->title_height)
     {
 	int i;
 	int nb = Scr->TBInfo.nleft + Scr->TBInfo.nright;
 
-	XSaveContext(dpy, tmp_win->title_w, TwmContext, (caddr_t) tmp_win);
-	XSaveContext(dpy, tmp_win->title_w, ScreenContext, (caddr_t) Scr);
+	XSaveContext(dpy, tmp_win->title_w, TwmContext, (XPointer) tmp_win);
+	XSaveContext(dpy, tmp_win->title_w, ScreenContext, (XPointer) Scr);
 	for (i = 0; i < nb; i++) {
 	    XSaveContext(dpy, tmp_win->titlebuttons[i].window, TwmContext,
-			 (caddr_t) tmp_win);
+			 (XPointer) tmp_win);
 	    XSaveContext(dpy, tmp_win->titlebuttons[i].window, ScreenContext,
-			 (caddr_t) Scr);
+			 (XPointer) Scr);
 	}
 	if (tmp_win->hilite_wl)
 	{
-	    XSaveContext(dpy, tmp_win->hilite_wl, TwmContext, (caddr_t)tmp_win);
-	    XSaveContext(dpy, tmp_win->hilite_wl, ScreenContext, (caddr_t)Scr);
+	    XSaveContext(dpy, tmp_win->hilite_wl, TwmContext, (XPointer)tmp_win);
+	    XSaveContext(dpy, tmp_win->hilite_wl, ScreenContext, (XPointer)Scr);
 	}
 	if (tmp_win->hilite_wr)
 	{
-	    XSaveContext(dpy, tmp_win->hilite_wr, TwmContext, (caddr_t)tmp_win);
-	    XSaveContext(dpy, tmp_win->hilite_wr, ScreenContext, (caddr_t)Scr);
+	    XSaveContext(dpy, tmp_win->hilite_wr, TwmContext, (XPointer)tmp_win);
+	    XSaveContext(dpy, tmp_win->hilite_wr, ScreenContext, (XPointer)Scr);
+	}
+	if (tmp_win->lolite_wl)
+	{
+	    XSaveContext(dpy, tmp_win->lolite_wl, TwmContext, (XPointer)tmp_win);
+	    XSaveContext(dpy, tmp_win->lolite_wl, ScreenContext, (XPointer)Scr);
+	}
+	if (tmp_win->lolite_wr)
+	{
+	    XSaveContext(dpy, tmp_win->lolite_wr, TwmContext, (XPointer)tmp_win);
+	    XSaveContext(dpy, tmp_win->lolite_wr, ScreenContext, (XPointer)Scr);
 	}
     }
 
@@ -972,6 +1253,7 @@ IconMgr *iconp;
     if (RootFunction)
 	ReGrab();
     WMapAddWindow (tmp_win);
+    SetPropsIfCaptiveCtwm (tmp_win);
     return (tmp_win);
 }
 
@@ -1060,25 +1342,32 @@ TwmWindow *tmp_win;
 		    GrabModeAsync, GrabModeAsync, None, 
 		    Scr->FrameCursor);
 	    
-    }
-}
-
-void ClickToFocusGrab (tmp_win)
-TwmWindow *tmp_win;
-{
-    if (tmp_win) {
-	XGrabButton (dpy, AnyButton, None, tmp_win->w, 
+	if (Scr->IgnoreLockModifier && !(tmp->mods & LockMask)) {
+	    XGrabButton (dpy, tmp->num, tmp->mods | LockMask, tmp_win->frame, 
 		    True, ButtonPressMask | ButtonReleaseMask,
 		    GrabModeAsync, GrabModeAsync, None, 
 		    Scr->FrameCursor);
+	}
     }
-}
-
-void ClickToFocusUngrab (tmp_win)
-TwmWindow *tmp_win;
-{
-    if (tmp_win) {
-	XUngrabButton (dpy, AnyButton, AnyModifier, tmp_win->w);
+    if (Scr->ClickToFocus) {
+	XGrabButton (dpy, AnyButton, None, tmp_win->w, 
+		    True, ButtonPressMask | ButtonReleaseMask,
+		    GrabModeSync, GrabModeAsync, None, 
+		    Scr->FrameCursor);
+	XGrabButton (dpy, AnyButton, LockMask, tmp_win->w, 
+		    True, ButtonPressMask | ButtonReleaseMask,
+		    GrabModeSync, GrabModeAsync, None, 
+		    Scr->FrameCursor);
+    } else
+    if (Scr->RaiseOnClick) {
+	XGrabButton (dpy, Scr->RaiseOnClickButton, None, tmp_win->w, 
+		    True, ButtonPressMask | ButtonReleaseMask,
+		    GrabModeSync, GrabModeAsync, None, 
+		    Scr->FrameCursor);
+	XGrabButton (dpy, Scr->RaiseOnClickButton, LockMask, tmp_win->w, 
+		    True, ButtonPressMask | ButtonReleaseMask,
+		    GrabModeSync, GrabModeAsync, None, 
+		    Scr->FrameCursor);
     }
 }
 
@@ -1110,29 +1399,49 @@ TwmWindow *tmp_win;
 	    if (tmp->mods & AltMask) break;
 	    XGrabKey(dpy, tmp->keycode, tmp->mods, tmp_win->w, True,
 		GrabModeAsync, GrabModeAsync);
+	    if (!Scr->IgnoreLockModifier || (tmp->mods & LockMask)) break;
+	    XGrabKey(dpy, tmp->keycode, tmp->mods | LockMask, tmp_win->w, True,
+		GrabModeAsync, GrabModeAsync);
 	    break;
 
 	case C_ICON:
-	    if (tmp_win->icon && tmp_win->icon->w)
-		XGrabKey(dpy, tmp->keycode, tmp->mods, tmp_win->icon->w, True,
+	    if (!tmp_win->icon || tmp_win->icon->w) break;
+	    XGrabKey(dpy, tmp->keycode, tmp->mods, tmp_win->icon->w, True,
+		    GrabModeAsync, GrabModeAsync);
+	    if (!Scr->IgnoreLockModifier || (tmp->mods & LockMask)) break;
+	    XGrabKey(dpy, tmp->keycode, tmp->mods | LockMask, tmp_win->icon->w, True,
 		    GrabModeAsync, GrabModeAsync);
 	    break;
 
 	case C_TITLE:
-	    if (tmp_win->title_w)
-		XGrabKey(dpy, tmp->keycode, tmp->mods, tmp_win->title_w, True,
+	    if (!tmp_win->title_w) break;
+	    XGrabKey(dpy, tmp->keycode, tmp->mods, tmp_win->title_w, True,
+		    GrabModeAsync, GrabModeAsync);
+	    if (!Scr->IgnoreLockModifier || (tmp->mods & LockMask)) break;
+	    XGrabKey(dpy, tmp->keycode, tmp->mods | LockMask, tmp_win->title_w, True,
 		    GrabModeAsync, GrabModeAsync);
 	    break;
 
 	case C_NAME:
 	    XGrabKey(dpy, tmp->keycode, tmp->mods, tmp_win->w, True,
 		GrabModeAsync, GrabModeAsync);
-	    if (tmp_win->icon && tmp_win->icon->w)
+	    if (Scr->IgnoreLockModifier && !(tmp->mods & LockMask))
+		XGrabKey(dpy, tmp->keycode, tmp->mods | LockMask, tmp_win->w, True,
+			GrabModeAsync, GrabModeAsync);
+	    if (tmp_win->icon && tmp_win->icon->w) {
 		XGrabKey(dpy, tmp->keycode, tmp->mods, tmp_win->icon->w, True,
 		    GrabModeAsync, GrabModeAsync);
-	    if (tmp_win->title_w)
+		if (Scr->IgnoreLockModifier && !(tmp->mods & LockMask))
+		    XGrabKey(dpy, tmp->keycode, tmp->mods | LockMask, tmp_win->icon->w,
+			True, GrabModeAsync, GrabModeAsync);
+	    }
+	    if (tmp_win->title_w) {
 		XGrabKey(dpy, tmp->keycode, tmp->mods, tmp_win->title_w, True,
 		    GrabModeAsync, GrabModeAsync);
+		if (Scr->IgnoreLockModifier && !(tmp->mods & LockMask))
+		    XGrabKey(dpy, tmp->keycode, tmp->mods | LockMask, tmp_win->title_w,
+			True, GrabModeAsync, GrabModeAsync);
+	    }
 	    break;
 	/*
 	case C_ROOT:
@@ -1149,11 +1458,29 @@ TwmWindow *tmp_win;
 	    for (p = Scr->iconmgr; p != NULL; p = p->next)
 	    {
 		XUngrabKey(dpy, tmp->keycode, tmp->mods, p->twm_win->w);
+		if (Scr->IgnoreLockModifier && !(tmp->mods & LockMask))
+		    XUngrabKey(dpy, tmp->keycode, tmp->mods | LockMask, p->twm_win->w);
 	    }
 	}
     }
 }
 
+void ComputeCommonTitleOffsets ()
+{
+    int buttonwidth = (Scr->TBInfo.width + Scr->TBInfo.pad);
+
+    Scr->TBInfo.leftx = Scr->TBInfo.rightoff = Scr->FramePadding;
+    if (Scr->TBInfo.nleft  > 0) Scr->TBInfo.leftx    += Scr->ButtonIndent;
+    if (Scr->TBInfo.nright > 0) Scr->TBInfo.rightoff += (Scr->ButtonIndent +
+			       (Scr->TBInfo.nright * buttonwidth) -
+				Scr->TBInfo.pad);
+
+    Scr->TBInfo.titlex = (Scr->TBInfo.leftx +
+				(Scr->TBInfo.nleft * buttonwidth) -
+				Scr->TBInfo.pad +
+				Scr->TitlePadding);
+}
+
 static void CreateHighlightWindows (tmp_win)
     TwmWindow *tmp_win;
 {
@@ -1247,23 +1574,71 @@ static void CreateHighlightWindows (tmp_win)
 		       Scr->d_visual, valuemask, &attributes);
 }
 
-
-void ComputeCommonTitleOffsets ()
+static void CreateLowlightWindows (tmp_win)
+    TwmWindow *tmp_win;
 {
-    int buttonwidth = (Scr->TBInfo.width + Scr->TBInfo.pad);
+    XSetWindowAttributes attributes;    /* attributes for create windows */
+    unsigned long valuemask;
+    int h = (Scr->TitleHeight - 2 * Scr->FramePadding);
+    int y = Scr->FramePadding;
+    ColorPair cp;
 
-    Scr->TBInfo.leftx = Scr->TBInfo.rightoff = Scr->FramePadding;
-    if (Scr->TBInfo.nleft  > 0) Scr->TBInfo.leftx    += Scr->ButtonIndent;
-    if (Scr->TBInfo.nright > 0) Scr->TBInfo.rightoff += (Scr->ButtonIndent +
-			       ((Scr->TBInfo.nright * buttonwidth) -
-				Scr->TBInfo.pad));
+    if (!Scr->UseSunkTitlePixmap || ! tmp_win->titlehighlight) {
+	tmp_win->lolite_wl = (Window) 0;
+	tmp_win->lolite_wr = (Window) 0;
+	return;
+    }
+    /*
+     * If a special highlight pixmap was given, use that.  Otherwise,
+     * use a nice, even gray pattern.  The old horizontal lines look really
+     * awful on interlaced monitors (as well as resembling other looks a
+     * little bit too closely), but can be used by putting
+     *
+     *                 Pixmaps { TitleHighlight "hline2" }
+     *
+     * (or whatever the horizontal line bitmap is named) in the startup
+     * file.  If all else fails, use the foreground color to look like a
+     * solid line.
+     */
 
-    Scr->TBInfo.titlex = (Scr->TBInfo.leftx +
-			  (Scr->TBInfo.nleft * buttonwidth) - Scr->TBInfo.pad +
-			  Scr->TitlePadding);
-    return;
+    if (! tmp_win->LoliteImage) {
+        if (Scr->HighlightPixmapName) {
+            cp = tmp_win->title;
+            cp.shadc = tmp_win->title.shadd;
+            cp.shadd = tmp_win->title.shadc;
+            tmp_win->LoliteImage = GetImage (Scr->HighlightPixmapName, cp);
+        }
+    }
+    if (tmp_win->LoliteImage) {
+        valuemask = CWBackPixmap;
+        attributes.background_pixmap = tmp_win->LoliteImage->pixmap;
+    } else {
+        valuemask = CWBackPixel;
+        attributes.background_pixel = tmp_win->title.fore;
+    }
+
+    if (Scr->use3Dtitles) {
+        y += 2;
+        h -= 4;
+    }
+    if (Scr->TitleJustification == J_LEFT)
+        tmp_win->lolite_wl = (Window) 0;
+    else
+        tmp_win->lolite_wl = XCreateWindow (dpy, tmp_win->title_w, 0, y,
+                       (unsigned int) Scr->TBInfo.width, (unsigned int) h,
+                       (unsigned int) 0, Scr->d_depth, (unsigned int) CopyFromParent,
+                       Scr->d_visual, valuemask, &attributes);
+
+    if (Scr->TitleJustification == J_RIGHT)
+        tmp_win->lolite_wr = (Window) 0;
+    else
+        tmp_win->lolite_wr = XCreateWindow (dpy, tmp_win->title_w, 0, y,
+                       (unsigned int) Scr->TBInfo.width, (unsigned int) h,
+                       (unsigned int) 0,  Scr->d_depth, (unsigned int) CopyFromParent,
+                       Scr->d_visual, valuemask, &attributes);
 }
 
+
 void ComputeWindowTitleOffsets (tmp_win, width, squeeze)
     TwmWindow *tmp_win;
     Bool squeeze;
@@ -1300,7 +1675,7 @@ void ComputeWindowTitleOffsets (tmp_win, width, squeeze)
     if (tmp_win->hilite_wr || Scr->TBInfo.nright > 0) 
       tmp_win->highlightxr += Scr->TitlePadding;
     tmp_win->rightx = width - Scr->TBInfo.rightoff;
-    if (squeeze && tmp_win->squeeze_info) {
+    if (squeeze && tmp_win->squeeze_info && !tmp_win->squeezed) {
 	int rx = (tmp_win->highlightxr + 
 		  (tmp_win->hilite_wr
 		    ? Scr->TBInfo.width * 2 : 0) +
@@ -1323,7 +1698,7 @@ void ComputeTitleLocation (tmp)
     tmp->title_x = tmp->frame_bw3D - tmp->frame_bw;
     tmp->title_y = tmp->frame_bw3D - tmp->frame_bw;
 
-    if (tmp->squeeze_info) {
+    if (tmp->squeeze_info && !tmp->squeezed) {
 	register SqueezeInfo *si = tmp->squeeze_info;
 	int basex;
 	int maxwidth = tmp->frame_width;
@@ -1379,8 +1754,10 @@ static void CreateWindowTitlebarButtons (tmp_win)
 
     if (tmp_win->title_height == 0)
     {
-	tmp_win->hilite_wl = 0;
-	tmp_win->hilite_wr = 0;
+	tmp_win->hilite_wl = (Window) 0;
+	tmp_win->hilite_wr = (Window) 0;
+	tmp_win->lolite_wl = (Window) 0;
+	tmp_win->lolite_wr = (Window) 0;
 	return;
     }
 
@@ -1446,9 +1823,12 @@ static void CreateWindowTitlebarButtons (tmp_win)
     }
 
     CreateHighlightWindows (tmp_win);
+    CreateLowlightWindows  (tmp_win);
     XMapSubwindows(dpy, tmp_win->title_w);
     if (tmp_win->hilite_wl) XUnmapWindow(dpy, tmp_win->hilite_wl);
     if (tmp_win->hilite_wr) XUnmapWindow(dpy, tmp_win->hilite_wr);
+    if (tmp_win->lolite_wl) XMapWindow(dpy, tmp_win->lolite_wl);
+    if (tmp_win->lolite_wr) XMapWindow(dpy, tmp_win->lolite_wr);
     return;
 }
 
@@ -1494,7 +1874,7 @@ CreateTwmColormap(c)
     TwmColormap *cmap;
     cmap = (TwmColormap *) malloc(sizeof(TwmColormap));
     if (!cmap ||
-	XSaveContext(dpy, c, ColormapContext, (caddr_t) cmap)) {
+	XSaveContext(dpy, c, ColormapContext, (XPointer) cmap)) {
 	if (cmap) free((char *) cmap);
 	return (NULL);
     }
@@ -1519,13 +1899,13 @@ CreateColormapWindow(w, creating_parent, property_window)
     cwin = (ColormapWindow *) malloc(sizeof(ColormapWindow));
     if (cwin) {
 	if (!XGetWindowAttributes(dpy, w, &attributes) ||
-	    XSaveContext(dpy, w, ColormapContext, (caddr_t) cwin)) {
+	    XSaveContext(dpy, w, ColormapContext, (XPointer) cwin)) {
 	    free((char *) cwin);
 	    return (NULL);
 	}
 
 	if (XFindContext(dpy, attributes.colormap,  ColormapContext,
-		(caddr_t *)&cwin->colormap) == XCNOENT) {
+		(XPointer *)&cwin->colormap) == XCNOENT) {
 	    cwin->colormap = cmap = CreateTwmColormap(attributes.colormap);
 	    if (!cmap) {
 		XDeleteContext(dpy, w, ColormapContext);
@@ -1637,7 +2017,7 @@ FetchWmColormapWindows (tmp)
 		 */
 		if (j == tmp->cmaps.number_cwins) {
 		    if (XFindContext(dpy, cmap_windows[i], ColormapContext,
-				     (caddr_t *)&cwins[i]) == XCNOENT) {
+				     (XPointer *)&cwins[i]) == XCNOENT) {
 			if ((cwins[i] = CreateColormapWindow(cmap_windows[i],
 				    (Bool) tmp->cmaps.number_cwins == 0,
 				    True)) == NULL) {
@@ -1661,7 +2041,7 @@ FetchWmColormapWindows (tmp)
 	number_cmap_windows = 1;
 
 	cwins = (ColormapWindow **) malloc(sizeof(ColormapWindow *));
-	if (XFindContext(dpy, tmp->w, ColormapContext, (caddr_t *)&cwins[0]) ==
+	if (XFindContext(dpy, tmp->w, ColormapContext, (XPointer *)&cwins[0]) ==
 		XCNOENT)
 	    cwins[0] = CreateColormapWindow(tmp->w,
 			    (Bool) tmp->cmaps.number_cwins == 0, False);
@@ -1751,3 +2131,231 @@ TwmWindow *t;
     }
     t->HiliteImage = image->next;
 }
+
+name_list **AddWindowRegion (geom, grav1, grav2)
+char *geom;
+int  grav1, grav2;
+{
+    WindowRegion *wr;
+    int mask;
+
+    wr = (WindowRegion*) malloc (sizeof (WindowRegion));
+    wr->next = NULL;
+
+    if (!Scr->FirstWindowRegion) Scr->FirstWindowRegion = wr;
+
+    wr->entries    = NULL;
+    wr->clientlist = NULL;
+    wr->grav1      = grav1;
+    wr->grav2      = grav2;
+    wr->x = wr->y = wr->w = wr->h = 0;
+
+    mask = XParseGeometry (geom, &wr->x, &wr->y, (unsigned int*) &wr->w,
+						 (unsigned int*) &wr->h);
+
+    if (mask & XNegative) wr->x += Scr->MyDisplayWidth  - wr->w;
+    if (mask & YNegative) wr->y += Scr->MyDisplayHeight - wr->h;
+
+    return (&(wr->clientlist));
+}
+
+void CreateWindowRegions () {
+    WindowRegion  *wr, *wr1 = NULL, *wr2 = NULL;
+    WorkSpaceList *wl;
+
+    for (wl = Scr->workSpaceMgr.workSpaceList; wl != NULL; wl = wl->next) {
+	wl->FirstWindowRegion = NULL;
+	wr2 = NULL;
+	for (wr = Scr->FirstWindowRegion; wr != NULL; wr = wr->next) {
+	    wr1  = (WindowRegion*) malloc (sizeof (WindowRegion));
+	    *wr1 = *wr;
+	    wr1->entries = (WindowEntry*) malloc (sizeof (WindowEntry));
+	    wr1->entries->next = 0;
+	    wr1->entries->x = wr1->x;
+	    wr1->entries->y = wr1->y;
+	    wr1->entries->w = wr1->w;
+	    wr1->entries->h = wr1->h;
+	    wr1->entries->twm_win = (TwmWindow*) 0;
+	    wr1->entries->used = 0;
+	    if (wr2) wr2->next = wr1; else wl->FirstWindowRegion = wr1;
+	    wr2 = wr1;
+	}
+	if (wr1) wr1->next = NULL;
+    }
+}
+
+
+Bool PlaceWindowInRegion (tmp_win, final_x, final_y)
+TwmWindow *tmp_win;
+int       *final_x, *final_y;
+{
+    WindowRegion  *wr;
+    WindowEntry	  *we;
+    int		   w, h;
+    WorkSpaceList *wl;
+
+    if (!Scr->FirstWindowRegion) return (False);
+    for (wl = Scr->workSpaceMgr.workSpaceList; wl != NULL; wl = wl->next) {
+	if (OCCUPY (tmp_win, wl)) break;
+    }
+    if (!wl) return (False);
+    w = tmp_win->frame_width;
+    h = tmp_win->frame_height;
+    we = (WindowEntry*) 0;
+    for (wr = wl->FirstWindowRegion; wr; wr = wr->next) {
+	if (LookInList (wr->clientlist, tmp_win->full_name, &tmp_win->class)) {
+	    for (we = wr->entries; we; we=we->next) {
+	        if (we->used) continue;
+	        if (we->w >= w && we->h >= h) break;
+	    }
+	    if (we) break;
+	}
+    }
+    tmp_win->wr = (WindowRegion*) 0;
+    if (!we) return (False);
+
+    splitWindowRegionEntry (we, wr->grav1, wr->grav2, w, h);
+    we->used = 1;
+    we->twm_win = tmp_win;
+    *final_x = we->x;
+    *final_y = we->y;
+    tmp_win->wr = wr;
+    return (True);
+}
+
+static void splitWindowRegionEntry (we, grav1, grav2, w, h)
+WindowEntry	*we;
+int		grav1, grav2;
+int		w, h;
+{
+    WindowEntry	*new;
+    int		save;
+
+    switch (grav1) {
+	case D_NORTH:
+	case D_SOUTH:
+	    save = we->w;
+	    if (w != we->w) splitWindowRegionEntry (we, grav2, grav1, w, we->h);
+	    if (h != we->h) {
+		new = (WindowEntry *) malloc (sizeof (WindowEntry));
+		new->twm_win = 0;
+		new->used = 0;
+		new->next = we->next;
+		we->next  = new;
+		new->x    = we->x;
+		new->h    = (we->h - h);
+		new->w    = we->w;
+		we->h     = h;
+		if (grav1 == D_SOUTH) {
+		    new->y = we->y;
+		    we->y  = new->y + new->h;
+		} else
+		    new->y = we->y + we->h;
+		}
+	    break;
+	case D_EAST:
+	case D_WEST:
+	    save = we->h;
+	    if (h != we->h) splitWindowRegionEntry (we, grav2, grav1, we->w, h);
+	    if (w != we->w) {
+		new = (WindowEntry *) malloc (sizeof (WindowEntry));
+		new->twm_win = 0;
+		new->used = 0;
+		new->next = we->next;
+		we->next  = new;
+		new->y    = we->y;
+		new->w    = (we->w - w);
+		new->h    = we->h;
+		we->w = w;
+		if (grav1 == D_EAST) {
+		    new->x = we->x;
+		    we->x  = new->x + new->w;
+		} else
+		    new->x = we->x + we->w;
+		}
+	    break;
+    }
+}
+
+static WindowEntry *findWindowEntry (wl, tmp_win, wrp)
+WorkSpaceList	*wl;
+TwmWindow	*tmp_win;
+WindowRegion	**wrp;
+{
+    WindowRegion *wr;
+    WindowEntry	 *we;
+
+    for (wr = wl->FirstWindowRegion; wr; wr = wr->next) {
+	for (we = wr->entries; we; we=we->next) {
+	    if (we->twm_win == tmp_win) {
+		if (wrp) *wrp = wr;
+		return we;
+	    }
+	}
+    }
+    return (WindowEntry*) 0;
+}
+
+static WindowEntry *prevWindowEntry (we, wr)
+WindowEntry	*we;
+WindowRegion	*wr;
+{
+    WindowEntry	*wp;
+
+    if (we == wr->entries) return 0;
+    for (wp = wr->entries; wp->next != we; wp=wp->next);
+    return wp;
+}
+
+static void mergeWindowEntries (old, we)
+WindowEntry	*old, *we;
+{
+    if (old->y == we->y) {
+	we->w = old->w + we->w;
+	if (old->x < we->x) we->x = old->x;
+    } else {
+	we->h = old->h + we->h;
+	if (old->y < we->y) we->y = old->y;
+    }
+}
+
+void RemoveWindowFromRegion (tmp_win)
+TwmWindow	*tmp_win;
+{
+    WindowEntry   *we, *wp, *wn;
+    WindowRegion  *wr;
+    WorkSpaceList *wl;
+
+    if (!Scr->FirstWindowRegion) return;
+    we = (WindowEntry*) 0;
+    for (wl = Scr->workSpaceMgr.workSpaceList; wl != NULL; wl = wl->next) {
+	we = findWindowEntry (wl, tmp_win, &wr);
+	if (we) break;
+    }
+    if (!we) return;
+
+    we->twm_win = 0;
+    we->used = 0;
+    wp = prevWindowEntry (we, wr);
+    wn = we->next;
+    for (;;) {
+	if (wp && wp->used == 0 &&
+	       ((wp->x == we->x && wp->w == we->w) ||
+	        (wp->y == we->y && wp->h == we->h))) {
+	    wp->next = we->next;
+	    mergeWindowEntries (we, wp);
+	    free ((char *) we);
+	    we = wp;
+	    wp = prevWindowEntry (wp, wr);
+	} else
+	if (wn && wn->used == 0 &&
+	       ((wn->x == we->x && wn->w == we->w) ||
+	        (wn->y == we->y && wn->h == we->h))) {
+	    we->next = wn->next;
+	    mergeWindowEntries (wn, we);
+	    free ((char *) wn);
+	    wn = we->next;
+	} else break;
+    }
+}
+
