@@ -62,6 +62,11 @@
 
 #include <stdio.h>
 #include "twm.h"
+#ifdef VMS
+#include <decw$include/Xatom.h>
+#else
+#include <X11/Xatom.h>
+#endif
 #include "util.h"
 #include "parse.h"
 #include "screen.h"
@@ -111,6 +116,8 @@ void CreateIconManagers()
     char *icon_name;
     WorkSpaceList *wlist;
     XWMHints	  wmhints;
+    XSizeHints	  sizehints;
+    int		  gravity;
     int bw;
 
     if (Scr->NoIconManagers)
@@ -141,9 +148,12 @@ void CreateIconManagers()
 	bw = LookInList (Scr->NoBorder, str, NULL) ? 0 :
 		(Scr->ThreeDBorderWidth ? Scr->ThreeDBorderWidth : Scr->BorderWidth);
 
-	if (mask & XNegative)
+	if (mask & XNegative) {
 	    JunkX += Scr->MyDisplayWidth - p->width - 2 * bw;
-
+	    gravity = (mask & YNegative) ? SouthEastGravity : NorthEastGravity;
+	} else {
+	    gravity = (mask & YNegative) ? SouthWestGravity : NorthWestGravity;
+	}
 	if (mask & YNegative)
 	    JunkY += Scr->MyDisplayHeight - p->height - 2 * bw;
 
@@ -162,9 +172,14 @@ void CreateIconManagers()
 	Scr->workSpaceMgr.activeWSPC = wlist;
 	wmhints.initial_state = NormalState;
 	wmhints.input         = True;
+	wmhints.flags         = InputHint | StateHint;
 	XSetWMHints (dpy, p->w, &wmhints);
 	p->twm_win = AddWindow(p->w, TRUE, p);
 
+	sizehints.flags       = PWinGravity;
+	sizehints.win_gravity = gravity;
+	XSetWMSizeHints (dpy, p->w, &sizehints, XA_WM_NORMAL_HINTS);
+
 	p->twm_win->mapped = FALSE;
 	SetMapStateProp (p->twm_win, WithdrawnState);
 	if (p->twm_win && p->twm_win->wmhints &&
@@ -387,6 +402,86 @@ void MoveIconManager(dir)
     }
 }
 
+/***********************************************************************
+ *
+ *  Procedure:
+ *	MoveMappedIconManager - move the pointer around in an icon manager
+ *
+ *  Inputs:
+ *	dir	- one of the following:
+ *			F_FORWMAPICONMGR	- forward in the window list
+ *			F_BACKMAPICONMGR	- backward in the window list
+ *
+ *  Special Considerations:
+ *	none
+ *
+ ***********************************************************************
+ */
+
+void MoveMappedIconManager(dir)
+    int dir;
+{
+    IconMgr *ip;
+    WList *tmp = NULL;
+    WList *orig = NULL;
+    int got_it;
+
+    if (!Current) return;
+
+    ip = Current->iconmgr;
+
+    got_it = 0;
+    tmp = Current;
+    orig = Current;
+
+    while(!got_it) {
+	switch(dir)
+	{
+	    case F_FORWMAPICONMGR:
+		if ((tmp = tmp->next) == NULL)
+		    tmp = ip->first;
+		break;
+
+	    case F_BACKMAPICONMGR:
+		if ((tmp = tmp->prev) == NULL)
+		    tmp = ip->last;
+		break;
+	}
+	if (tmp->twm->mapped) {
+	    got_it = 1;
+	    break;
+	}
+	if (tmp == orig)
+	    break;
+    }
+
+    if (!got_it)
+    {
+	fprintf (stderr, "%s:  unable to find open window in icon manager\n", 
+		 ProgramName);
+	return;
+    }
+
+    if (tmp == NULL)
+      return;
+
+    /* raise the frame so the icon manager is visible */
+    if (ip->twm_win->mapped) {
+	RaiseWindow(ip->twm_win);
+	XWarpPointer(dpy, None, tmp->icon, 0,0,0,0, 5, 5);
+    } else {
+	if (tmp->twm->title_height) {
+	    int tbx = Scr->TBInfo.titlex;
+	    int x = tmp->twm->highlightxr;
+	    XWarpPointer (dpy, None, tmp->twm->title_w, 0, 0, 0, 0,
+			  tbx + (x - tbx) / 2,
+			  Scr->TitleHeight / 4);
+	} else {
+	    XWarpPointer (dpy, None, tmp->twm->w, 0, 0, 0, 0, 5, 5);
+	}
+    }
+}
+
 /***********************************************************************
  *
  *  Procedure:
@@ -569,19 +664,19 @@ WList *AddIconManager(tmp_win)
     if (Scr->WindowMask) XRaiseWindow (dpy, Scr->WindowMask);
     XMapWindow(dpy, tmp->w);
 
-    XSaveContext(dpy, tmp->w, IconManagerContext, (caddr_t) tmp);
-    XSaveContext(dpy, tmp->w, TwmContext, (caddr_t) tmp_win);
-    XSaveContext(dpy, tmp->w, ScreenContext, (caddr_t) Scr);
-    XSaveContext(dpy, tmp->icon, TwmContext, (caddr_t) tmp_win);
-    XSaveContext(dpy, tmp->icon, ScreenContext, (caddr_t) Scr);
+    XSaveContext(dpy, tmp->w, IconManagerContext, (XPointer) tmp);
+    XSaveContext(dpy, tmp->w, TwmContext, (XPointer) tmp_win);
+    XSaveContext(dpy, tmp->w, ScreenContext, (XPointer) Scr);
+    XSaveContext(dpy, tmp->icon, TwmContext, (XPointer) tmp_win);
+    XSaveContext(dpy, tmp->icon, ScreenContext, (XPointer) Scr);
 
     if (!ip->twm_win->isicon)
     {
-      if (OCCUPY (ip->twm_win, Scr->workSpaceMgr.activeWSPC)) {
-	SetMapStateProp (ip->twm_win, NormalState);
-	XMapWindow (dpy, ip->w);
-	XMapWindow (dpy, ip->twm_win->frame);
-      }
+	if (OCCUPY (ip->twm_win, Scr->workSpaceMgr.activeWSPC)) {
+	    SetMapStateProp (ip->twm_win, NormalState);
+	    XMapWindow (dpy, ip->w);
+	    XMapWindow (dpy, ip->twm_win->frame);
+	}
 	ip->twm_win->mapped = TRUE;
     }
     tmp->nextv = old;
@@ -849,7 +944,7 @@ void PackIconManager(ip)
     int new_x, new_y;
     int savewidth;
     WList *tmp;
-    int mask, bw;
+    int mask;
     unsigned int JunkW, JunkH;
 
     if (Scr->use3Diconmanagers) {
