@@ -189,8 +189,38 @@ extern Bool PopUpMenu();
 extern void MakeWorkspacesMenu ();
 extern MenuRoot *FindMenuRoot();
 extern Bool AddFuncKey();
+extern Bool AddFuncButton();
+extern int DestroyMenu ();
+extern int PopDownMenu();
+extern int HideMenu();
 extern int ExecuteFunction();
 extern int DeferExecution();
+extern int NeedToDefer();
+extern int ReGrab();
+extern int CreateTitleButton();
+extern int PaintEntry();
+extern int Paint3DEntry();
+extern int PaintNormalEntry();
+extern int PaintMenu();
+extern int UpdateMenu();
+extern int MakeMenus();
+extern int MakeMenu();
+extern int MoveMenu();
+extern int DeIconify();
+extern int Iconify();
+extern int WarpToScreen();
+extern int BumpWindowColormap();
+extern int SetMapStateProp();
+extern int SendDeleteWindowMessage ();
+extern int SendSaveYourselfMessage ();
+extern int SendTakeFocusMessage ();
+extern int FindConstraint ();
+extern MosaicFade ();
+extern WarpCursorToDefaultEntry ();
+extern Bool GetWMState();
+extern void PlaceTransients();
+extern void MakeWorkspacesMenu ();
+extern void ModifyCurrentTB();
 extern void Execute();
 extern void ShowIconManager ();
 extern void HideIconManager ();
@@ -203,6 +233,15 @@ extern void RaiseFrame();
 extern void FocusOnRoot();
 extern void TryToPack ();
 extern void TryToPush ();
+extern void TryToGrid ();
+extern void resizeFromCenter();
+extern void WarpAlongRing ();
+extern void WarpToWindow ();
+extern void DisplayPosition ();
+extern void packwindow ();
+extern void fillwindow ();
 extern Boolean TryNotToMoveOff ();
+extern void AutoSqueeze ();
+extern void Squeeze();
 
 #endif /* _MENUS_ */
