@@ -68,7 +68,7 @@
  ***********************************************************************/
 
 #include <stdio.h>
-#if defined(sony_news)
+#if defined(sony_news) || defined __QNX__
 #  include <ctype.h>
 #endif
 #ifdef VMS
@@ -85,6 +85,7 @@
 #include "util.h"
 #include "gram.h"
 #include "parse.h"
+#include "version.h"
 #ifdef VMS
 #include <decw$include/Xatom.h> 
 #else
@@ -99,6 +100,10 @@
 #include <netdb.h>
 #endif
 
+#if defined(ultrix)
+#define NOSTEMP
+#endif
+
 #ifndef SYSTEM_INIT_FILE
 #ifdef VMS
 #define SYSTEM_INIT_FILE "DECW$SYSTEM_DEFAULTS:SYSTEM.CTWMRC"
@@ -108,10 +113,9 @@
 #endif
 #define BUF_LEN 300
 
-static int ParseJustification ();
-static int ParseAlignement ();
 static int ParseRandomPlacement ();
 static int ParseButtonStyle ();
+extern int yyparse(),twmrc_error_prefix();
 
 static FILE *twmrc;
 static int ptr = 0;
@@ -142,6 +146,9 @@ int (*twmInputFunc)();
 
 extern char *defTwmrc[];		/* default bindings */
 
+extern int captive;
+extern char *captivename;
+
 #ifdef SOUNDS
 extern int set_sound_host();
 #endif
@@ -451,17 +458,20 @@ static int twmFileInput()
 
 static int m4twmFileInput()
 {
-        if (overflowlen) return((int) overflowbuff[--overflowlen]);
+    int pppp;
+    if (overflowlen){
+	return((int) overflowbuff[--overflowlen]);
+    }
 
-        while (ptr == len) {
-                if (fgets(buff, BUF_LEN, twmrc) == NULL) return(0);
-
-                yylineno++;
-
-                ptr = 0;
-                len = strlen(buff);
-        }
-        return ((int)buff[ptr++]);
+    while (ptr == len) {
+	if (fgets(buff, BUF_LEN, twmrc) == NULL) return(0);
+	
+	yylineno++;
+	
+	ptr = 0;
+	len = strlen(buff);
+    }
+    return ((int)buff[ptr++]);
 }
 #endif /* USEM4 */
 
@@ -584,6 +594,18 @@ typedef struct _TwmKeyword {
 #define kw0_Use3DWMap			47
 #define kw0_ReverseCurrentWorkspace	48
 #define kw0_DontWarpCursorInWMap	49
+#define kw0_CenterFeedbackWindow	50
+#define kw0_WarpToDefaultMenuEntry	51
+#define kw0_SchrinkIconTitles		52
+#define kw0_AutoRaiseIcons		53
+#define kw0_use3DIconBorders		54
+#define kw0_UseSunkTitlePixmap          55
+#define kw0_ShortAllWindowsMenus	56
+#define kw0_RaiseWhenAutoUnSqueeze	57
+#define kw0_RaiseOnClick		58
+#define kw0_IgnoreLockModifier		59
+#define kw0_AutoFocusToTransients       60 /* kai */
+#define kw0_PackNewWindows		61
 
 #define kws_UsePPosition		1
 #define kws_IconFont			2
@@ -604,6 +626,7 @@ typedef struct _TwmKeyword {
 #define kws_SoundHost			16
 #endif
 #define kws_WMgrButtonStyle		17
+#define kws_WorkSpaceFont               18
 
 #define kwn_ConstrainedMoveTime		1
 #define kwn_MoveDelta			2
@@ -633,6 +656,10 @@ typedef struct _TwmKeyword {
 #define kwn_MenuShadowDepth		26
 #define kwn_IconManagerShadowDepth	27
 #define kwn_MovePackResistance		28
+#define kwn_XMoveGrid			29
+#define kwn_YMoveGrid			30
+#define kwn_OpenWindowTimeout		31
+#define kwn_RaiseOnClickButton		32
 
 #define kwcl_BorderColor		1
 #define kwcl_IconManagerHighlight	2
@@ -668,9 +695,12 @@ static TwmKeyword keytable[] = {
     { "alwaysontop",		ALWAYS_ON_TOP, 0 },
     { "alwaysshowwindowwhenmovingfromworkspacemanager", KEYWORD, kw0_ShowWinWhenMovingInWmgr },
     { "animationspeed",		NKEYWORD, kwn_AnimationSpeed },
+    { "autofocustotransients",  KEYWORD, kw0_AutoFocusToTransients }, /* kai */
     { "autooccupy",		KEYWORD, kw0_AutoOccupy },
     { "autoraise",		AUTO_RAISE, 0 },
+    { "autoraiseicons",		KEYWORD, kw0_AutoRaiseIcons },
     { "autorelativeresize",	KEYWORD, kw0_AutoRelativeResize },
+    { "autosqueeze",		AUTOSQUEEZE, 0 },
     { "benicetocolormap",	KEYWORD, kw0_BeNiceToColormap },
     { "bordercolor",		CLKEYWORD, kwcl_BorderColor },
     { "borderresizecursors",	KEYWORD, kw0_BorderResizeCursors },
@@ -682,6 +712,7 @@ static TwmKeyword keytable[] = {
     { "buttonindent",		NKEYWORD, kwn_ButtonIndent },
     { "c",			CONTROL, 0 },
     { "center",			JKEYWORD, J_CENTER },
+    { "centerfeedbackwindow",	KEYWORD, kw0_CenterFeedbackWindow },
     { "changeworkspacefunction", CHANGE_WORKSPACE_FUNCTION, 0 },
     { "clearshadowcontrast",	NKEYWORD, kwn_ClearShadowContrast },
     { "clicktofocus",		KEYWORD, kw0_ClickToFocus },
@@ -700,6 +731,7 @@ static TwmKeyword keytable[] = {
     { "donticonifybyunmapping",	DONT_ICONIFY_BY_UNMAPPING, 0 },
     { "dontmoveoff",		KEYWORD, kw0_DontMoveOff },
     { "dontpaintrootwindow",	KEYWORD, kw0_DontPaintRootWindow },
+    { "dontsetinactive",	DONTSETINACTIVE, 0 },
     { "dontsqueezetitle",	DONT_SQUEEZE_TITLE, 0 },
     { "dontwarpcursorinwmap",	KEYWORD, kw0_DontWarpCursorInWMap },
     { "east",			DKEYWORD, D_EAST },
@@ -710,6 +742,7 @@ static TwmKeyword keytable[] = {
     { "f.altkeymap",		FSKEYWORD, F_ALTKEYMAP },
     { "f.autoraise",		FKEYWORD, F_AUTORAISE },
     { "f.backiconmgr",		FKEYWORD, F_BACKICONMGR },
+    { "f.backmapiconmgr",	FKEYWORD, F_BACKMAPICONMGR },
     { "f.beep",			FKEYWORD, F_BEEP },
     { "f.bottomzoom",		FKEYWORD, F_BOTTOMZOOM },
     { "f.circledown",		FKEYWORD, F_CIRCLEDOWN },
@@ -719,15 +752,18 @@ static TwmKeyword keytable[] = {
     { "f.cutfile",		FKEYWORD, F_CUTFILE },
     { "f.deiconify",		FKEYWORD, F_DEICONIFY },
     { "f.delete",		FKEYWORD, F_DELETE },
+    { "f.deleteordestroy",	FKEYWORD, F_DELETEORDESTROY },
     { "f.deltastop",		FKEYWORD, F_DELTASTOP },
     { "f.destroy",		FKEYWORD, F_DESTROY },
     { "f.downiconmgr",		FKEYWORD, F_DOWNICONMGR },
     { "f.downworkspace",	FKEYWORD, F_DOWNWORKSPACE },
     { "f.exec",			FSKEYWORD, F_EXEC },
     { "f.file",			FSKEYWORD, F_FILE },
+    { "f.fill",			FSKEYWORD, F_FILL },
     { "f.focus",		FKEYWORD, F_FOCUS },
     { "f.forcemove",		FKEYWORD, F_FORCEMOVE },
     { "f.forwiconmgr",		FKEYWORD, F_FORWICONMGR },
+    { "f.forwmapiconmgr",	FKEYWORD, F_FORWMAPICONMGR },
     { "f.fullzoom",		FKEYWORD, F_FULLZOOM },
     { "f.function",		FSKEYWORD, F_FUNCTION },
     { "f.gotoworkspace",	FSKEYWORD, F_GOTOWORKSPACE },
@@ -736,6 +772,7 @@ static TwmKeyword keytable[] = {
     { "f.hideworkspacemgr",	FKEYWORD, F_HIDEWORKMGR },
     { "f.horizoom",		FKEYWORD, F_HORIZOOM },
     { "f.htzoom",		FKEYWORD, F_TOPZOOM },
+    { "f.hypermove",		FKEYWORD, F_HYPERMOVE },
     { "f.hzoom",		FKEYWORD, F_HORIZOOM },
     { "f.iconify",		FKEYWORD, F_ICONIFY },
     { "f.identify",		FKEYWORD, F_IDENTIFY },
@@ -753,6 +790,7 @@ static TwmKeyword keytable[] = {
     { "f.nop",			FKEYWORD, F_NOP },
     { "f.occupy",		FKEYWORD, F_OCCUPY },
     { "f.occupyall",		FKEYWORD, F_OCCUPYALL },
+    { "f.pack",			FSKEYWORD, F_PACK },
     { "f.pin",			FKEYWORD, F_PIN },
     { "f.previconmgr",		FKEYWORD, F_PREVICONMGR },
     { "f.prevworkspace",	FKEYWORD, F_PREVWORKSPACE },
@@ -767,9 +805,11 @@ static TwmKeyword keytable[] = {
 #endif
     { "f.resize",		FKEYWORD, F_RESIZE },
     { "f.restart",		FKEYWORD, F_RESTART },
+    { "f.restoregeometry",	FKEYWORD, F_RESTOREGEOMETRY },
     { "f.righticonmgr",		FKEYWORD, F_RIGHTICONMGR },
     { "f.rightworkspace",	FKEYWORD, F_RIGHTWORKSPACE },
     { "f.rightzoom",		FKEYWORD, F_RIGHTZOOM },
+    { "f.savegeometry",		FKEYWORD, F_SAVEGEOMETRY },
     { "f.saveyourself",		FKEYWORD, F_SAVEYOURSELF },
     { "f.separator",		FKEYWORD, F_SEPARATOR },
     { "f.setbuttonsstate",	FKEYWORD, F_SETBUTTONSTATE },
@@ -780,6 +820,7 @@ static TwmKeyword keytable[] = {
     { "f.sorticonmgr",		FKEYWORD, F_SORTICONMGR },
     { "f.source",		FSKEYWORD, F_BEEP },  /* XXX - don't work */
     { "f.speedupanimation",	FKEYWORD, F_SPEEDUPANIMATION },
+    { "f.squeeze",		FKEYWORD, F_SQUEEZE },
     { "f.startanimation",	FKEYWORD, F_STARTANIMATION },
     { "f.stopanimation",	FKEYWORD, F_STOPANIMATION },
     { "f.title",		FKEYWORD, F_TITLE },
@@ -788,6 +829,7 @@ static TwmKeyword keytable[] = {
     { "f.togglesound",		FKEYWORD, F_TOGGLESOUND },
 #endif
     { "f.togglestate",		FKEYWORD, F_TOGGLESTATE },
+    { "f.toggleworkspacemgr",	FKEYWORD, F_TOGGLEWORKMGR },
     { "f.topzoom",		FKEYWORD, F_TOPZOOM },
     { "f.trace",		FSKEYWORD, F_TRACE },
     { "f.twmrc",		FKEYWORD, F_RESTART },
@@ -834,6 +876,7 @@ static TwmKeyword keytable[] = {
     { "iconregionalignement",	SKEYWORD, kws_IconRegionAlignement },
     { "iconregionjustification",SKEYWORD, kws_IconRegionJustification },
     { "icons",			ICONS, 0 },
+    { "ignorelockmodifier",	KEYWORD, kw0_IgnoreLockModifier },
     { "interpolatemenucolors",	KEYWORD, kw0_InterpolateMenuColors },
     { "l",			LOCK, 0 },
     { "left",			JKEYWORD, J_LEFT },
@@ -892,10 +935,15 @@ static TwmKeyword keytable[] = {
     { "opaquemovethreshold",	NKEYWORD, kwn_OpaqueMoveThreshold },
     { "opaqueresize",		OPAQUERESIZE, 0 },
     { "opaqueresizethreshold",	NKEYWORD, kwn_OpaqueResizeThreshold },
+    { "openwindowtimeout",	NKEYWORD, kwn_OpenWindowTimeout },
+    { "packnewwindows",		KEYWORD, kw0_PackNewWindows },
     { "pixmapdirectory",	SKEYWORD, kws_PixmapDirectory },
     { "pixmaps",		PIXMAPS, 0 },
     { "r",			ROOT, 0 },
     { "raisedelay",		NKEYWORD, kwn_RaiseDelay },
+    { "raiseonclick",		KEYWORD, kw0_RaiseOnClick },
+    { "raiseonclickbutton",	NKEYWORD, kwn_RaiseOnClickButton },
+    { "raisewhenautounsqueeze",	KEYWORD, kw0_RaiseWhenAutoUnSqueeze },
     { "randomplacement",	SKEYWORD, kws_RandomPlacement },
     { "reallymoveinworkspacemanager",	KEYWORD, kw0_ReallyMoveInWorkspaceManager },
     { "resize",			RESIZE, 0 },
@@ -907,8 +955,10 @@ static TwmKeyword keytable[] = {
     { "root",			ROOT, 0 },
     { "s",			SHIFT, 0 },
     { "savecolor",              SAVECOLOR, 0},
+    { "schrinkicontitles",	KEYWORD, kw0_SchrinkIconTitles },
     { "select",			SELECT, 0 },
     { "shift",			SHIFT, 0 },
+    { "shortallwindowsmenus",	KEYWORD, kw0_ShortAllWindowsMenus },
     { "showiconmanager",	KEYWORD, kw0_ShowIconManager },
     { "showworkspacemanager",	KEYWORD, kw0_ShowWorkspaceManager },
     { "smarticonify",		KEYWORD, kw0_SmartIconify },
@@ -920,6 +970,7 @@ static TwmKeyword keytable[] = {
     { "squeezetitle",		SQUEEZE_TITLE, 0 },
     { "starticonified",		START_ICONIFIED, 0 },
     { "startinmapstate",	KEYWORD, kw0_StartInMapState },
+    { "startsqueezed",		STARTSQUEEZED, 0 },
     { "stayupmenus",		KEYWORD, kw0_StayUpMenus },
     { "sunkfocuswindowtitle",	KEYWORD, kw0_SunkFocusWindowTitle },
     { "t",			TITLE, 0 },
@@ -937,8 +988,11 @@ static TwmKeyword keytable[] = {
     { "transienthasoccupation",	KEYWORD, kw0_TransientHasOccupation },
     { "transientontop",		NKEYWORD, kwn_TransientOnTop },
     { "unknownicon",		SKEYWORD, kws_UnknownIcon },
+    { "unmapbymovingfaraway",	UNMAPBYMOVINGFARAWAY, 0 },
     { "usepposition",		SKEYWORD, kws_UsePPosition },
+    { "usesunktitlepixmap",     KEYWORD, kw0_UseSunkTitlePixmap },
     { "usethreedborders",	KEYWORD, kw0_Use3DBorders },
+    { "usethreediconborders",	KEYWORD, kw0_use3DIconBorders },
     { "usethreediconmanagers",	KEYWORD, kw0_Use3DIconManagers },
     { "usethreedmenus",		KEYWORD, kw0_Use3DMenus },
     { "usethreedtitles",	KEYWORD, kw0_Use3DTitles },
@@ -947,20 +1001,25 @@ static TwmKeyword keytable[] = {
     { "wait",			WAITC, 0 },
     { "warpcursor",		WARP_CURSOR, 0 },
     { "warpringonscreen",	KEYWORD, kw0_WarpRingOnScreen },
+    { "warptodefaultmenuentry",	KEYWORD, kw0_WarpToDefaultMenuEntry },
     { "warpunmapped",		KEYWORD, kw0_WarpUnmapped },
     { "west",			DKEYWORD, D_WEST },
     { "window",			WINDOW, 0 },
     { "windowfunction",		WINDOW_FUNCTION, 0 },
+    { "windowregion",		WINDOW_REGION, 0 },
     { "windowring",		WINDOW_RING, 0 },
     { "wmgrbuttonshadowdepth",	NKEYWORD, kwn_WMgrButtonShadowDepth },
     { "wmgrbuttonstyle",	SKEYWORD, kws_WMgrButtonStyle },
     { "wmgrhorizbuttonindent",	NKEYWORD, kwn_WMgrHorizButtonIndent },
     { "wmgrvertbuttonindent",	NKEYWORD, kwn_WMgrVertButtonIndent },
     { "workspace", 		WORKSPACE, 0 },
+    { "workspacefont",          SKEYWORD, kws_WorkSpaceFont },
     { "workspacemanagergeometry", WORKSPCMGR_GEOMETRY, 0 },
     { "workspaces",             WORKSPACES, 0},
+    { "xmovegrid",		NKEYWORD, kwn_XMoveGrid },
     { "xorvalue",		NKEYWORD, kwn_XorValue },
     { "xpmicondirectory",	SKEYWORD, kws_PixmapDirectory },
+    { "ymovegrid",		NKEYWORD, kwn_YMoveGrid },
     { "zoom",			ZOOM, 0 },
 };
 
@@ -1104,6 +1163,10 @@ int do_single_keyword (keyword)
 	Scr->DontPaintRootWindow = TRUE;
 	return 1;
 
+      case kw0_UseSunkTitlePixmap:
+	Scr->UseSunkTitlePixmap = TRUE;
+	return 1;
+
       case kw0_Use3DBorders:
 	Scr->use3Dborders = TRUE;
 	return 1;
@@ -1183,6 +1246,52 @@ int do_single_keyword (keyword)
       case kw0_DontWarpCursorInWMap:
 	Scr->DontWarpCursorInWMap = TRUE;
 	return 1;
+
+      case kw0_CenterFeedbackWindow:
+	Scr->CenterFeedbackWindow = TRUE;
+	return 1;
+
+      case kw0_WarpToDefaultMenuEntry:
+	Scr->WarpToDefaultMenuEntry = TRUE;
+	return 1;
+
+      case kw0_SchrinkIconTitles:
+	Scr->SchrinkIconTitles = TRUE;
+	return 1;
+
+      case kw0_AutoRaiseIcons:
+	Scr->AutoRaiseIcons = TRUE;
+	return 1;
+
+      /* kai */
+      case kw0_AutoFocusToTransients:
+        Scr->AutoFocusToTransients = TRUE;
+        return 1;
+
+      case kw0_use3DIconBorders:
+	Scr->use3Diconborders = TRUE;
+	return 1;
+
+      case kw0_ShortAllWindowsMenus:
+	Scr->ShortAllWindowsMenus = TRUE;
+	return 1;
+
+      case kw0_RaiseWhenAutoUnSqueeze:
+	Scr->RaiseWhenAutoUnSqueeze = TRUE;
+	return 1;
+
+      case kw0_RaiseOnClick:
+	Scr->RaiseOnClick = TRUE;
+	return 1;
+
+      case kw0_IgnoreLockModifier:
+	Scr->IgnoreLockModifier = TRUE;
+	return 1;
+
+      case kw0_PackNewWindows:
+	Scr->PackNewWindows = TRUE;
+	return 1;
+
     }
     return 0;
 }
@@ -1218,6 +1327,31 @@ int do_string_keyword (keyword, s)
 	    return 1;
 	}
 
+#ifdef I18N
+      case kws_IconFont:
+	if (!Scr->HaveFonts) Scr->IconFont.basename = s;
+	return 1;
+
+      case kws_ResizeFont:
+	if (!Scr->HaveFonts) Scr->SizeFont.basename = s;
+	return 1;
+
+      case kws_MenuFont:
+	if (!Scr->HaveFonts) Scr->MenuFont.basename = s;
+	return 1;
+
+      case kws_WorkSpaceFont:
+	if (!Scr->HaveFonts) Scr->workSpaceMgr.workspaceWindow.windowFont.basename = s;
+	return 1;
+
+      case kws_TitleFont:
+	if (!Scr->HaveFonts) Scr->TitleBarFont.basename = s;
+	return 1;
+
+      case kws_IconManagerFont:
+	if (!Scr->HaveFonts) Scr->IconManagerFont.basename = s;
+	return 1;
+#else	
       case kws_IconFont:
 	if (!Scr->HaveFonts) Scr->IconFont.name = s;
 	return 1;
@@ -1230,6 +1364,10 @@ int do_string_keyword (keyword, s)
 	if (!Scr->HaveFonts) Scr->MenuFont.name = s;
 	return 1;
 
+      case kws_WorkSpaceFont:
+	if (!Scr->HaveFonts) Scr->workSpaceMgr.workspaceWindow.windowFont.name = s;
+	return 1;
+
       case kws_TitleFont:
 	if (!Scr->HaveFonts) Scr->TitleBarFont.name = s;
 	return 1;
@@ -1237,17 +1375,18 @@ int do_string_keyword (keyword, s)
       case kws_IconManagerFont:
 	if (!Scr->HaveFonts) Scr->IconManagerFont.name = s;
 	return 1;
+#endif
 
       case kws_UnknownIcon:
 	if (Scr->FirstTime) GetUnknownIcon (s);
 	return 1;
 
       case kws_IconDirectory:
-	if (Scr->FirstTime) Scr->IconDirectory = ExpandFilename (s);
+	if (Scr->FirstTime) Scr->IconDirectory = ExpandFilePath (s);
 	return 1;
 
       case kws_PixmapDirectory:
-	if (Scr->FirstTime) Scr->PixmapDirectory = ExpandFilename (s);
+	if (Scr->FirstTime) Scr->PixmapDirectory = ExpandFilePath (s);
 	return 1;
 
       case kws_MaxWindowSize:
@@ -1366,6 +1505,18 @@ int do_number_keyword (keyword, num)
 	Scr->MovePackResistance = num;
 	return 1;
 
+      case kwn_XMoveGrid:
+	if (num <   1) num =   1;
+	if (num > 100) num = 100;
+	Scr->XMoveGrid = num;
+	return 1;
+
+      case kwn_YMoveGrid:
+	if (num <   1) num =   1;
+	if (num > 100) num = 100;
+	Scr->YMoveGrid = num;
+	return 1;
+
       case kwn_XorValue:
 	if (Scr->FirstTime) Scr->XORvalue = num;
 	return 1;
@@ -1477,6 +1628,17 @@ int do_number_keyword (keyword, num)
 	if (Scr->MenuShadowDepth < 0) Scr->MenuShadowDepth = 2;
 	return 1;
 
+      case kwn_OpenWindowTimeout:
+	if (Scr->FirstTime) Scr->OpenWindowTimeout = num;
+	if (Scr->OpenWindowTimeout < 0) Scr->OpenWindowTimeout = 0;
+	return 1;
+
+      case kwn_RaiseOnClickButton:
+	if (Scr->FirstTime) Scr->RaiseOnClickButton = num;
+	if (Scr->RaiseOnClickButton < 1) Scr->RaiseOnClickButton = 1;
+	if (Scr->RaiseOnClickButton > MAX_BUTTONS) Scr->RaiseOnClickButton = MAX_BUTTONS;
+	return 1;
+
     }
 
     return 0;
@@ -1620,6 +1782,7 @@ int do_string_savecolor(colormode, s)
   Pixel p;
   GetColor(colormode, &p, s);
   put_pixel_on_root(p);
+  return 0;
 }
 
 /*
@@ -1642,6 +1805,7 @@ int key;
     cpnew = (Cptr)malloc(sizeof(Cnode));
     cpnew->i = key; cpnew->next = NULL; cptrav->next = cpnew;
   }
+  return 0;
 }
 
 /*
@@ -1715,13 +1879,15 @@ static int ParseRandomPlacement (s)
     return (-1);
 }
 
-static int ParseJustification (s)
+int ParseJustification (s)
     register char *s;
 {
     if (strlen (s) == 0) return (-1);
     if (strcmp (s, "default") == 0) return J_CENTER;
+    if (strcmp (s,   "undef") == 0) return J_UNDEF;
     XmuCopyISOLatin1Lowered (s, s);
 
+    if (strcmp (s,   "undef") == 0) return J_UNDEF;
     if (strcmp (s, "default") == 0) return J_CENTER;
     if (strcmp (s,    "left") == 0) return J_LEFT;
     if (strcmp (s,  "center") == 0) return J_CENTER;
@@ -1730,13 +1896,15 @@ static int ParseJustification (s)
     return (-1);
 }
 
-static int ParseAlignement (s)
+int ParseAlignement (s)
     register char *s;
 {
     if (strlen (s) == 0) return (-1);
     if (strcmp (s, "default") == 0) return J_CENTER;
+    if (strcmp (s,   "undef") == 0) return J_UNDEF;
     XmuCopyISOLatin1Lowered (s, s);
 
+    if (strcmp (s,   "undef") == 0) return J_UNDEF;
     if (strcmp (s, "default") == 0) return J_CENTER;
     if (strcmp (s,     "top") == 0) return J_TOP;
     if (strcmp (s,  "center") == 0) return J_CENTER;
@@ -1862,7 +2030,7 @@ FILE *fraw;
 /* Code taken and munged from xrdb.c */
 #define MAXHOSTNAME 255
 #define Resolution(pixels, mm)  ((((pixels) * 100000 / (mm)) + 50) / 100)
-#define EXTRA   12
+#define EXTRA   16
 
 static char *MkDef(name, def)
 char *name, *def;
@@ -1884,36 +2052,15 @@ char *name, *def;
                 fprintf(stderr, "Can't get %d bytes for arg parm\n", n);
                 exit(468);
         }
-        strcpy(cp, "define(");
-        strcpy(cp+7, name);
-        *(cp + nl + 7) = ',';
-        *(cp + nl + 8) = ' ';
-        strcpy(cp + nl + 9, def);
-        strcat(cp + nl + 9, ")\n");
+	strcpy (cp, "define(`");
+	strcat (cp, name);
+	strcat (cp, "', `");
+	strcat (cp, def);
+	strcat (cp, "')\n");
 
         return(cp);
 }
 
-static char *MkQte(name, def)
-char *name, *def;
-{
-        char *cp, *cp2;
-
-        cp = malloc(2 + strlen(def));
-        if (cp == NULL) {
-                fprintf(stderr, "Can't get %d bytes for arg parm\n", 2 + strlen(
-def));
-                exit(469);
-        }
-        *cp = '\"';
-        strcpy(cp + 1, def);
-        strcat(cp, "\"");
-        cp2 = MkDef(name, cp);
-        free(cp);               /* Not really needed, but good habits die hard..
-. */
-        return(cp2);
-}
-
 static char *MkNum(name, def)
 char *name;
 int def;
@@ -1942,7 +2089,6 @@ Display *display;
 char *host;
 {
         extern int KeepTmpFile;
-        int i;
         Screen *screen;
         Visual *visual;
         char client[MAXHOSTNAME], server[MAXHOSTNAME], *colon;
@@ -1993,6 +2139,7 @@ char *host;
         fputs(MkNum("PLANES",DisplayPlanes(display, Scr->screen)), tmpf);
         fputs(MkNum("BITS_PER_RGB", visual->bits_per_rgb), tmpf);
         fputs(MkDef("TWM_TYPE", "ctwm"), tmpf);
+        fputs(MkDef("TWM_VERSION", VersionNumber), tmpf);
         switch(visual->class) {
                 case(StaticGray):       vc = "StaticGray";      break;
                 case(GrayScale):        vc = "GrayScale";       break;
@@ -2011,6 +2158,12 @@ char *host;
 #ifdef XPM
 	fputs(MkDef("XPM", "Yes"), tmpf);
 #endif
+	if (captive && captivename) {
+            fputs (MkDef ("TWM_CAPTIVE", "Yes"), tmpf);
+            fputs (MkDef ("TWM_CAPTIVE_NAME", captivename), tmpf);
+	} else {
+            fputs (MkDef ("TWM_CAPTIVE", "No"), tmpf);
+	}
         if (KeepTmpFile) {
                 fprintf(stderr, "Left file: %s\n", tmp_name);
         } else {
