@@ -202,11 +202,16 @@ typedef struct ScreenInfo
     WorkSpaceMgr workSpaceMgr;
     short	workSpaceManagerActive;
     name_list	*OccupyAll;	/* list of window names occupying all workspaces at startup */
+    name_list	*UnmapByMovingFarAway;
+    name_list	*DontSetInactive;
+    name_list	*AutoSqueeze;
+    name_list	*StartSqueezed;
     short 	use3Dmenus;
     short 	use3Dtitles;
     short 	use3Diconmanagers;
     short 	use3Dborders;
     short 	use3Dwmap;
+    short	use3Diconborders;
     short	SunkFocusWindowTitle;
     short	WMgrVertButtonIndent;
     short	WMgrHorizButtonIndent;
@@ -222,6 +227,13 @@ typedef struct ScreenInfo
     short	ShowWinWhenMovingInWmgr;
     short	ReverseCurrentWorkspace;
     short	DontWarpCursorInWMap;
+    short	XMoveGrid, YMoveGrid;
+    short	FastServer;
+    short	CenterFeedbackWindow;
+    short	SchrinkIconTitles;
+    short	AutoRaiseIcons;
+    short       AutoFocusToTransients; /* kai */
+    short       PackNewWindows;
 
     name_list *BorderColorL;
     name_list *IconBorderColorL;
@@ -278,6 +290,7 @@ typedef struct ScreenInfo
     IconMgr *iconmgr;		/* default icon manager  */
     struct IconRegion *FirstRegion;	/* pointer to icon regions */
     struct IconRegion *LastRegion;	/* pointer to the last icon region */
+    struct WindowRegion *FirstWindowRegion;	/* pointer to window regions */
     char *IconDirectory;	/* icon directory to search */
     char *PixmapDirectory;	/* Pixmap directory to search */
     int SizeStringOffset;	/* x offset in size window for drawing */
@@ -296,6 +309,7 @@ typedef struct ScreenInfo
     short AutoRaiseDefault;	/* AutoRaise all windows if true */
     short NoDefaults;		/* do not add in default UI stuff */
     short UsePPosition;		/* what do with PPosition, see values below */
+    short UseSunkTitlePixmap;
     short AutoRelativeResize;	/* start resize relative to position in quad */
     short FocusRoot;		/* is the input focus on the root ? */
     short WarpCursor;		/* warp cursor on de-iconify ? */
@@ -339,6 +353,7 @@ typedef struct ScreenInfo
     short Shadow;		/* show the menu shadow */
     short InterpolateMenuColors;/* make pretty menus */
     short StayUpMenus;		/* stay up menus */
+    short WarpToDefaultMenuEntry; /* warp cursor to default menu entry, if any  */
     short ClickToFocus;		/* click to focus */
     short NoIconManagers;	/* Don't create any icon managers */
     short ClientBorderWidth;	/* respect client window border width */
@@ -349,6 +364,12 @@ typedef struct ScreenInfo
     short WarpUnmapped;		/* allow warping to unmapped windows */
     short WindowRingAll;	/* add all windows to the ring */
     short WarpRingAnyWhere;	/* warp to ring even if window is not visible */
+    short ShortAllWindowsMenus;	/* Eliminates Icon and Workspace Managers */
+    short OpenWindowTimeout;	/* Timeout when a window tries to open */
+    short RaiseWhenAutoUnSqueeze;
+    short RaiseOnClick;		/* Raise a window when clieked into */
+    short RaiseOnClickButton;		/* Raise a window when clieked into */
+    short IgnoreLockModifier;	/* Should we ignore the lock modifier */
 
     FuncKey FuncKeyRoot;
     FuncButton FuncButtonRoot;
