@@ -177,10 +177,22 @@ typedef SIGNAL_T (*SigProc)();	/* type of function returned by signal() */
     Gcv.background = fix_back;\
     XChangeGC(dpy, Scr->NormalGC, GCForeground|GCBackground,&Gcv)
 
+#ifdef I18N
+#define MaxSize(a, b)  (((a) < (b)) ? (b) : (a))
+#define MinSize(a, b)  (((a) > (b)) ? (b) : (a))
+#endif
+
 typedef struct MyFont
 {
+#ifdef I18N    
+    char *basename;			/* name of the font */
+    XFontSet	font_set;
+    int         ascent;
+    int         descent;
+#else    
     char *name;			/* name of the font */
-    XFontStruct *font;		/* font structure */
+    XFontStruct	*font;		/* font structure */
+#endif
     int height;			/* height of the font */
     int y;			/* Y coordinate to draw characters */
 } MyFont;
@@ -221,6 +233,7 @@ typedef struct _SqueezeInfo {
     int denom;				/* 0 for pix count or denominator */
 } SqueezeInfo;
 
+#define J_UNDEF			0
 #define J_LEFT			1
 #define J_CENTER		2
 #define J_RIGHT			3
@@ -262,26 +275,20 @@ typedef struct Colormaps
 #define ColormapsScoreboardLength(cm) ((cm)->number_cwins * \
 				       ((cm)->number_cwins - 1) / 2)
 
-typedef enum {match_none, match_class, match_name, match_icon} Matchtype;
+typedef struct WindowRegion {
+    struct WindowRegion	*next;
+    int			x, y, w, h;
+    int			grav1, grav2;
+    name_list           *clientlist;
+    struct WindowEntry	*entries;
+} WindowRegion;
 
-typedef struct Icon
-{
-    Matchtype	match;
-    Window	w;		/* the icon window */
-    Window	bm_w;		/* the icon bitmap window */
-    Image	*image;		/* image icon structure */
-    int		x;		/* icon text x coordinate */
-    int		y;		/* icon text y coordiante */
-    int		w_width;	/* width of the icon window */
-    int		w_height;	/* height of the icon window */
-    int		width;          /* width of the icon bitmap */
-    int		height;         /* height of the icon bitmap */
-    char	*pattern;	/* Why this icon was choosed */
-    Pixel	border;		/* border color */
-    ColorPair	iconc;
-    Bool	has_title;
-    int		border_width;
-} Icon;
+typedef struct WindowEntry {
+    struct WindowEntry	*next;
+    int			x, y, w, h;
+    struct TwmWindow	*twm_win;
+    short 		used;
+} WindowEntry;
 
 /* for each window that is on the display, one of these structures
  * is allocated and linked into a list 
@@ -296,9 +303,11 @@ typedef struct TwmWindow
     Window title_w;		/* the title bar window */
     Window hilite_wl;		/* the left hilite window */
     Window hilite_wr;		/* the right hilite window */
+    Window lolite_wl;		/* the right lolite window */
+    Window lolite_wr;		/* the right lolite window */
     Cursor curcurs;		/* current resize cursor */
     Pixmap gray;
-    Icon *icon;			/* the curent icon */
+    struct Icon *icon;		/* the curent icon */
     name_list *iconslist;	/* the current list of icons */
     int frame_x;		/* x position of frame */
     int frame_y;		/* y position of frame */
@@ -306,6 +315,10 @@ typedef struct TwmWindow
     int frame_height;		/* height of frame */
     int frame_bw;		/* borderwidth of frame */
     int frame_bw3D;		/* 3D borderwidth of frame */
+    int actual_frame_x;		/* save frame_y of frame when squeezed */
+    int actual_frame_y;		/* save frame_x of frame when squeezed */
+    int actual_frame_width;	/* save width of frame when squeezed */
+    int actual_frame_height;	/* save height of frame when squeezed */
     int title_x;
     int title_y;
     int title_height;		/* height of the title bar */
@@ -334,6 +347,7 @@ typedef struct TwmWindow
     short isicon;		/* is the window an icon now ? */
     short icon_on;		/* is the icon visible */
     short mapped;		/* is the window mapped ? */
+    short squeezed;		/* is the window squeezed ? */
     short auto_raise;		/* should we auto-raise this window ? */
     short forced;		/* has had an icon forced upon it */
     short icon_not_ours;	/* icon pixmap or window supplied to us */
@@ -365,11 +379,57 @@ typedef struct TwmWindow
 
     short OpaqueMove;
     short OpaqueResize;
+    short UnmapByMovingFarAway;
+    short AutoSqueeze;
+    short StartSqueezed;
+    short DontSetInactive;
     Bool hasfocusvisible;	/* The window has visivle focus*/
     int  occupation;
     Image *HiliteImage;                /* focus highlight window background */
+    Image *LoliteImage;                /* focus lowlight window background */
+    WindowRegion *wr;
+    struct {
+	int x, y;
+	int width, height;
+    } savegeometry;
+
+#ifdef X11R6
+    Bool nameChanged;	/* did WM_NAME ever change? */
+    /* did the user ever change the width/height? {yes, no, or unknown} */
+    Bool widthEverChangedByUser;
+    Bool heightEverChangedByUser;
+#endif
+
 } TwmWindow;
 
+#ifdef X11R6
+typedef struct TWMWinConfigEntry
+{
+    struct TWMWinConfigEntry *next;
+    int tag;
+    char *client_id;
+    char *window_role;
+    XClassHint class;
+    char *wm_name;
+    int wm_command_count;
+    char **wm_command;
+    short x, y;
+    unsigned short width, height;
+    short icon_x, icon_y;
+    Bool iconified;
+    Bool icon_info_present;
+    Bool width_ever_changed_by_user;
+    Bool height_ever_changed_by_user;
+   /* ===================[ Matthew McNeill Feb 1997 ]======================= *
+    * Added this property to facilitate restoration of workspaces when 
+    * restarting a session.
+    */
+    int occupation;
+   /* ====================================================================== */
+
+} TWMWinConfigEntry;
+#endif
+
 #define DoesWmTakeFocus		(1L << 0)
 #define DoesWmSaveYourself	(1L << 1)
 #define DoesWmDeleteWindow	(1L << 2)
@@ -382,11 +442,16 @@ typedef struct TwmWindow
 #define TBPM_MENU ":menu"	/* name of titlebar pixmap for menus */
 #define TBPM_QUESTION ":question"	/* name of unknown titlebar pixmap */
 
+#define TBPM_3DICONIFY ":xpm:iconify"
+#define TBPM_3DSUNKEN_RESIZE ":xpm:sunkresize"
+#define TBPM_3DBOX ":xpm:box"
+
 #define TBPM_3DDOT ":xpm:dot"		/* name of titlebar pixmap for dot */
 #define TBPM_3DRESIZE ":xpm:resize"	/* name of titlebar pixmap for resize button */
 #define TBPM_3DMENU ":xpm:menu"	/* name of titlebar pixmap for menus */
 #define TBPM_3DZOOM ":xpm:zoom"
 #define TBPM_3DBAR ":xpm:bar"
+#define TBPM_3DVBAR ":xpm:vbar"
 
 #ifndef X11R4
 #ifdef VMS
@@ -398,15 +463,22 @@ typedef struct TwmWindow
 #ifndef X_NOT_STDC_ENV
 #include <stdlib.h>
 #else
+#ifdef VMS
+#include <stdlib.h>
+#else
 extern char *malloc(), *calloc(), *realloc(), *getenv();
 extern void free();
 #endif
+#endif
 extern void Reborder();
 extern SIGNAL_T Done();
 void ComputeCommonTitleOffsets();
 void ComputeWindowTitleOffsets(), ComputeTitleLocation();
 extern char *ProgramName;
 extern Display *dpy;
+#ifdef X11R6
+  extern XtAppContext appContext;
+#endif
 extern Window ResizeWindow;	/* the window we are resizing */
 extern int HasShape;		/* this server supports Shape extension */
 
@@ -437,7 +509,7 @@ extern int JunkX;
 extern int JunkY;
 extern unsigned int JunkWidth, JunkHeight, JunkBW, JunkDepth, JunkMask;
 extern XGCValues Gcv;
-extern int InfoLines;
+extern int InfoLines,InfoWidth,InfoHeight;
 extern char Info[][INFO_SIZE];
 extern int Argc;
 extern char **Argv;
@@ -464,8 +536,18 @@ extern Atom _XA_WM_PROTOCOLS;
 extern Atom _XA_WM_TAKE_FOCUS;
 extern Atom _XA_WM_SAVE_YOURSELF;
 extern Atom _XA_WM_DELETE_WINDOW;
+extern Atom _XA_WM_CLIENT_MACHINE;
+#ifdef X11R6
+  extern Atom _XA_SM_CLIENT_ID;
+  extern Atom _XA_WM_CLIENT_LEADER;
+  extern Atom _XA_WM_WINDOW_ROLE;
+#endif
 
 #define OCCUPY(w, b) ((b == NULL) ? 1 : (w->occupation & (1 << b->number)))
-#define VISIBLE(w) (w->occupation & Scr->workSpaceMgr.visibility)
+#define VISIBLE(w) OCCUPY(w, Scr->workSpaceMgr.activeWSPC)
+
+#ifdef X11R4
+#define XPointer caddr_t
+#endif
 
 #endif /* _TWM_ */
