@@ -50,5 +50,10 @@
  * Author:  Claude Lecommandeur [ lecom@sic.epfl.ch ][ April 1992 ]
  */
 
-char *Version = "MIT X Consortium, R6, ctwm 3.3";
-
+#ifdef I18N
+char *Version = "MIT X Consortium, R6, ctwm 3.5 + I18N Patch ";
+char *VersionNumber = "3.5";
+#else
+char *Version = "MIT X Consortium, R6, ctwm 3.5";
+char *VersionNumber = "3.5";
+#endif
